from framework.ui_elements.ImagePresenter import ImagePresenter
from framework.ui_elements.AudioPresenter import AudioPresenter
import time

#
# This is a reward logic that displays a pop-up icon and gives simple audio feedback. 
#

class VisualRewardLogic:
    def __init__(self,
                 initial_score=0,                                                       # the initial score
                 score_image_params = {'scale':0.3,'pos':[-3.1,0.5,1],'clearafter':2},  # properties of the score response icon
                 score_sound_params = {'direction':-0.7,'volume':0.5},                  # properties of the score response sound
                 score_responses = {'high-gain':[25,'happy_star.png','xDingLing.wav'],  # mapping from reward tag to score update, picture and sound to play
                                    'low-gain':[5,'star.png','ding.wav'],
                                    'low-loss':[-5,'worried_smiley.png','xBuzz01.wav'],
                                    'high-loss':[-25,'sad_smiley.png','slap.wav']},
                 raise_speed = 0.5                                                      # speed at which the image raises
                 ):

        self.score = initial_score
        self.score_image = ImagePresenter(**score_image_params)
        self.score_sound = AudioPresenter(**score_sound_params)
        self.score_responses = score_responses
        self.raise_speed = raise_speed
        self.anim_time = None
        taskMgr.add(self._update_scoreimage,'RewardLogic.update_scoreimage()')
    
    def score_event(self,eventtype):
        """ Score handling function."""
        print eventtype
        outcome = self.score_responses[eventtype]
        self.score = self.score + outcome[0]
        self.score_image.submit(outcome[1])        
        self.score_sound.submit(outcome[2])        
        self.anim_time = time.time()

    def _update_scoreimage(self,task):
        if self.anim_time is not None:            
            self.score_image.icon.setPos(self.score_image.pos[0],self.score_image.pos[1],self.score_image.pos[2]+self.raise_speed*(time.time()-self.anim_time))
        return task.cont
            
    def destroy(self):
        taskMgr.remove('RewardLogic.update_scoreimage()')

