# -*- coding:utf-8 -*-
from framework.latentmodule import LatentModule 

class StimulusStream(LatentModule):
    """Base class for all stimulus streams; allows sending a message to some presenter."""
    def __init__(self):
        LatentModule.__init__(self)
        
    def submit(self,message,presenter,locktime=None,block=True,retryinterval=0.1,clearafter=None):
        """
        Submit a message to a presenter, optionally blocking until the presenter is available.
        The presenter may be locked upon successful submission for a specified amount of time.
        
        Returns if True if the submission was successful and false otherwise (always True if blocking).
        """
        if block:
            while not presenter.submit(message,locktime,clearafter):
                self.sleep(retryinterval)
            return True
        else:
            return presenter.submit(message,locktime,clearafter)
        